/* callback.c --- code of the different callbacks  */

/* Copyright (c) E. Lassauge, 2000.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.
 *
 * mailto:lassauge@mail.dotcom.fr
 * http://lassauge.free.fr/
 *
 * REVISION HISTORY:
 *
 */

#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "sms.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

void on_script_ok_pressed(GtkWidget * widget, GtkFileSelection *filesel)
{
    gchar      *filename;
    gchar      *ext;
    PProvider_t Pv=NULL;

    filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
    gtk_widget_destroy(GTK_WIDGET(filesel));
    ext = (gchar *)g_extension_pointer(filename);
    if (g_strncasecmp(g_extension_pointer(filename),SMSSEND_SUF,3) == 0)
    {
	Pv = create_provider(filename);
	if (!Pv)
	{
	    g_printerr(_("Error for '%s'!\n"),filename);
	}
    }
    g_free(filename);
}

void on_pref_ok_pressed(GtkButton * button, gpointer user_data)
{
    /* update preferences variables: user_data 1 if write pref file */
    GtkWidget *widget = GTK_WIDGET(button);
    gchar     *config_file;

    UseProxy = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(widget, "use_proxy")));

    if (UseProxy)
    {
	g_snprintf(ProxyHost, P_LENGTH, "%s", gtk_entry_get_text(GTK_ENTRY(lookup_widget(widget, "proxy_host"))));
	ProxyPort = (gint) atoi(gtk_entry_get_text(GTK_ENTRY(lookup_widget(widget, "proxy_port"))));

	g_snprintf(ProxyUser, P_LENGTH, "%s", gtk_entry_get_text(GTK_ENTRY(lookup_widget(widget, "proxy_user"))));
	g_snprintf(ProxyPass, P_LENGTH, "%s", gtk_entry_get_text(GTK_ENTRY(lookup_widget(widget, "proxy_password"))));
    }

    DebugLevel = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(widget, "debug_level")));
    TimeOut = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(widget, "timeout")));
    AutoCheck  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(widget, "auto_check")));
    KeepPass   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(widget, "keep_pass")));

    /* Save current configuration */
    config_file=gnome_util_prepend_user_home(".smssend/smssendrc");
    if ((int)user_data)
    {
	gchar *path;
	/* =/home/me/.smssend/smssendrc=/Proxy/use_proxy=1 */
	path = g_strdup_printf("=%s=/Proxy/use_proxy",config_file);
	gnome_config_set_bool(path,UseProxy);
	g_free(path);
	if (UseProxy)
	{
	    gchar *temp;

	    path = g_strdup_printf("=%s=/Proxy/proxy_host",config_file);
	    gnome_config_set_string(path,ProxyHost);
	    g_free(path);
	    path = g_strdup_printf("=%s=/Proxy/proxy_port",config_file);
	    gnome_config_set_int(path,ProxyPort);
	    g_free(path);
	    path = g_strdup_printf("=%s=/Proxy/proxy_user",config_file);
	    gnome_config_set_string(path,ProxyUser);
	    g_free(path);
	    path = g_strdup_printf("=%s=/Proxy/proxy_password",config_file);
	    temp = g_strdup(ProxyPass);
	    crypt_password(temp);
	    gnome_config_set_string(path,temp);
	    g_free(temp);
	    g_free(path);
	}
	path = g_strdup_printf("=%s=/Misc/debug_level",config_file);
	gnome_config_set_int(path,DebugLevel);
	g_free(path);
	path = g_strdup_printf("=%s=/Misc/timeout",config_file);
	gnome_config_set_int(path,TimeOut);
	g_free(path);
	path = g_strdup_printf("=%s=/Misc/auto_check",config_file);
	gnome_config_set_bool(path,AutoCheck);
	g_free(path);
	path = g_strdup_printf("=%s=/Misc/keep_pass",config_file);
	gnome_config_set_bool(path,KeepPass);
	g_free(path);

	gnome_config_sync();
    }
    g_free(config_file);
}

void on_pref_cancel_pressed(GtkButton * button, gpointer user_data)
{
    /* NOPE */
}

void on_new_file_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    /* Open a dialog box to choose a new provider file */
    scripts=create_scripts();
    gtk_widget_show(scripts);
}

void on_goto_smssend_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    /* Open a web browser on the smssend homepage */
    gnome_url_show(SMSSEND_URL_SCRIPTS);
}

void on_close_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    /* remove currently selected notebook tab */
    GtkNotebook *nb = GTK_NOTEBOOK(notebook);
    gint        curpage = gtk_notebook_get_current_page(nb);
    GtkWidget   *curwidget;
    PProvider_t   Pv;

    /* check page number ! */
    if (curpage < 0)
	return;
    /* get Provider struct from selected tab (saved at creation) */
    curwidget = nb->cur_page->child;
    Pv = (PProvider_t) gtk_object_get_data(GTK_OBJECT(curwidget),"provider");
    /* free Pv */
    DeleteProvider(Pv);

    gtk_notebook_remove_page(nb,curpage);
    numprovider--;
}

void on_exit_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    /* this is the end TODO: wait for end of current connection */
    gtk_exit(0);
}

void on_preferences_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    /* open the preference dialog box */
    preferences = create_preferences();
    gtk_widget_show(preferences);
}

void on_check_updates_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    /* launch "smssend -update" */
    launch_sms(NULL, FALSE, TRUE);
}

void on_about_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    /* open the about dialog box */
    about = create_about();
    gtk_widget_show(about);
}

void on_send_pressed(GtkButton * button, gpointer user_data)
{
    /* lauch smsend for the current provider */
    PProvider_t Pv = (PProvider_t) user_data;
    launch_sms(Pv, FALSE, FALSE);
}

void on_message_changed(GtkEditable * editable, gpointer user_data)
{
    /* check length of message with respect to the maximum size */

    guint Size = (guint) (user_data);	/* user_data yields Size field */
    GtkText *message = GTK_TEXT(editable);
    guint CurSize = gtk_text_get_length(message);
    GtkWidget *SizeVal = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(message), "remain");	/* remaining entry widget is saved in editable widget */

    if (CurSize >= Size)
    {
	/* check if not too much data: erase it */
	gtk_text_set_point(message, CurSize);
	gtk_text_backward_delete(message, (CurSize - Size));
	CurSize = gtk_text_get_length(message);
    }
    if ((Size - CurSize) < 0)
	return;
    if (SizeVal != (GtkWidget *) NULL)
    {
	/* update remaining */
	char remain[4];
	g_snprintf(remain, 4, "%d", Size - CurSize);
	gtk_entry_set_text(GTK_ENTRY(SizeVal), remain);
    }
}
