/* main.c --- main function for gtksmssend */

/* Copyright (c) E. Lassauge, 2000.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.
 *
 * mailto:lassauge@mail.dotcom.fr
 * http://lassauge.free.fr/
 *
 * REVISION HISTORY:
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "sms.h"
#include "interface.h"
#include "support.h"

int main(int argc, char *argv[])
{
    gchar *config_file;
    gchar *path;

#ifdef ENABLE_NLS
    bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
    textdomain(PACKAGE);
#endif

    gnome_init(PACKAGE, VERSION, argc, argv);

    /* init threads */
    g_thread_init(NULL);

    gnome_smssend = create_gnome_smssend();
    gtk_widget_show(gnome_smssend);

    /*
     * Get configuration values
     */
    config_file=gnome_util_prepend_user_home(".smssend/smssendrc");
    path = g_strdup_printf("=%s=/Proxy/use_proxy",config_file);
    UseProxy = gnome_config_get_bool_with_default(path,FALSE);
    g_free(path);
    if (UseProxy)
    {
	gchar *value;
	path = g_strdup_printf("=%s=/Proxy/proxy_host",config_file);
	value = gnome_config_get_string_with_default(path,NULL);
	if (value && strlen(value))
	    g_snprintf(ProxyHost,P_LENGTH,"%s",value);
	g_free(path);
	g_free(value);
	path = g_strdup_printf("=%s=/Proxy/proxy_port",config_file);
	ProxyPort = gnome_config_get_int_with_default(path,0);
	g_free(path);
	path = g_strdup_printf("=%s=/Proxy/proxy_user",config_file);
	value = gnome_config_get_string_with_default(path,NULL);
	if (value && strlen(value))
	    g_snprintf(ProxyUser,P_LENGTH,"%s",value);
	g_free(path);
	g_free(value);
	path = g_strdup_printf("=%s=/Proxy/proxy_password",config_file);
	value = gnome_config_get_string_with_default(path,NULL);
	if (value && strlen(value))
	{
	    crypt_password(value);
	    g_snprintf(ProxyPass,P_LENGTH,"%s",value);
	}
	g_free(path);
	g_free(value);
    }
    path = g_strdup_printf("=%s=/Misc/debug_level",config_file);
    DebugLevel = gnome_config_get_int_with_default(path,0);
    g_free(path);
    path = g_strdup_printf("=%s=/Misc/timeout",config_file);
    TimeOut = gnome_config_get_int_with_default(path,0);
    g_free(path);
    path = g_strdup_printf("=%s=/Misc/auto_check",config_file);
    AutoCheck = gnome_config_get_bool_with_default(path,FALSE);
    g_free(path);
    if (AutoCheck)
	gnome_url_show(SMSSEND_URL_SCRIPTS);
    path = g_strdup_printf("=%s=/Misc/keep_pass",config_file);
    KeepPass = gnome_config_get_bool_with_default(path,FALSE);
    g_free(path);
    g_free(config_file);

    /*
     * Check for script files and create notebook entries
     */
    create_notebook();
    gtk_main();
    return 0;
}
