/* sms.c --- code for management of script files: parts stolen from smssend  */

/* Copyright (c) E. Lassauge, 2000.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.
 *
 * mailto:lassauge@mail.dotcom.fr
 * http://lassauge.free.fr/
 *
 * REVISION HISTORY:
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <pthread.h>
#include <gnome.h>
#include <signal.h>

#include "sms.h"
#include "interface.h"
#include "support.h"

/* global preferences values */
gboolean UseProxy = FALSE;
gchar ProxyHost[P_LENGTH];
gint ProxyPort = 8080;
gchar ProxyUser[P_LENGTH];
gchar ProxyPass[P_LENGTH];
gint DebugLevel = 0;
gint TimeOut = 0;
gboolean AutoCheck = FALSE;
gboolean KeepPass = FALSE;

/* executable name for smssend */
#define SMSEXECNAME	"smssend"
#ifdef THREAD_SMSSEND
extern int main_smssend(int argc,char *argv[]);
#endif

/* parameters for DoSms() thread */
typedef struct
{
    PProvider_t Pv;	/* these 3 options can't be set together */
    gboolean    Help;
    gboolean    Update;
    gint        Status; /* TODO: use this fields for thread communication */
    gchar*      Message;
}
cmdParam_t, *PcmdParam_t;

static void *DoSms(void *arg);
const char arg_help[]   = "-help";
const char arg_update[] = "-update";
const char arg_minus[]  = "--";

void DeleteProvider(PProvider_t Pv)
{
    /* Free a provider struct: based on smssend code */
    int i;

    if (Pv->Params != NULL)
    {
	for (i = 0; i < Pv->NbParams; i++)
	{
	    if (Pv->Params[i].Name != NULL)
		g_free(Pv->Params[i].Name);
	    if (Pv->Params[i].Value != NULL)
		g_free(Pv->Params[i].Value);
	}
	free(Pv->Params);
    }
    if (Pv->Label != NULL)
	g_free(Pv->Label);
    if (Pv->Filename != NULL)
	g_free(Pv->Filename);
    g_free(Pv);
}

PProvider_t CreateProviderFromFile(const char FileName[])
{
    /* Load a provider struct from script file: based on smssend code */
    FILE *fp;
    char Name[1024], Value[1024], Saf[1024], *Str;
    PProvider_t Pv;
    int NbParams;

    fp = fopen(FileName, "r");
    if (fp == NULL)
	return NULL;

    Pv = g_new0(Provider_t, 1);
    NbParams = 0;

    while (SU_ParseConfig(fp, Name, sizeof(Name), Value, sizeof(Value)))
      {
	  if (g_strncasecmp(Name, "NbParams",9) == 0)
	    {
		Pv->NbParams = atoi(Value);
		Pv->Params = g_new0(Param_t, Pv->NbParams);
	    }
	  else if (Name[0] == '%')
	    {
		if (NbParams >= Pv->NbParams)
		  {
		      g_printerr(_("SmsSend Error in provider loader : More than NbParams has been found\n"));
		      DeleteProvider(Pv);
		      return NULL;
		  }
		Pv->Params[NbParams].Name = g_strdup(Name + 1);
		if (Value[0] != 0)
		  {
		      if (Value[0] == ':')
			{
			    Pv->Params[NbParams].Help = g_strdup(SU_TrimLeft(Value + 1));
			}
		      strcpy(Saf, Value);
		      Str = strtok(Value, " ");
		      while (Str != NULL)
			{
			    if (g_strcasecmp(Str, "Hidden") == 0)
				Pv->Params[NbParams].Hidden = 1;
			    else if (g_strcasecmp(Str, "Convert") == 0)
				Pv->Params[NbParams].Convert = 1;
			    else if (g_strncasecmp(Str, "Size", 4) == 0)
				Pv->Params[NbParams].Size = atoi(Str + 5);
			    else if (Str[0] == ':')
			      {
				  Str = strchr(Saf, ':');
				  Pv->Params[NbParams].Help = g_strdup(SU_TrimLeft(Str + 1));
				  break;
			      }
			    else
				g_printerr(_("Unknown option in Param values : %s\n"), Str);
			    Str = strtok(NULL, " ");
			}
		  }
		NbParams++;
	    }
      }

    fclose(fp);

    return Pv;
}

/* TODO:
 * fix updating of progressbar and message: it seems that it's not
 * possible within a thread. Update sms_param with the return codes and
 * messages.
 * On the other hand add a timeout: gtk_timeout_add (1000, update_bar, NULL);
 * which will use a list of PcmdParam_t pointers (or an array) to update
 * the appbar depending on the tab selected.
 * We need to attach a callback to the tab selection to update the appbar:
 *      "switch-page" signal.
 * And to check this sms_param when deleting a provider tab:
 *      on_close_activate() callback.
 * Protect update and reading of the struct with a mutex:
 *      g_mutex_lock(&bar_mutex);
 *      // read or update values
 *      g_mutex_unlock (&bar_mutex);
 * Maybe use a single mutex or one with each provider:
 *      GMutex* g_mutex_new(); // when creating a Provider
 *      g_mutex_free (); // in on_close_activate() callback
 */

/* temporary defines to eliminate appbar updating */
#define gnome_appbar_set_status(a,b)
#define gnome_appbar_set_progress(a,b)

static void *DoSms(void *arg)
{
    /* thread to call smssend in a fork->exec */
    PcmdParam_t sms_param = (PcmdParam_t) arg;
    char  proxy[P_LENGTH+7] = "";
    char  pass[P_LENGTH*2+3] = "";
    char  debug[10] = "";
    char  timeout[5] = "";
#ifdef THREAD_SMSSEND
    int   smsstatus = 0;
#else
    pid_t smsid = -1;
#endif
    char  *args[20];
    static int   retval=0;
#ifndef gnome_appbar_set_status
    GnomeAppBar *appbar = GNOME_APPBAR(lookup_widget(gnome_smssend, "appbar"));
#endif

    pthread_detach (pthread_self ());
    if (UseProxy)
    {
	  /* update proxy args */
	  g_snprintf(proxy, P_LENGTH+8, "-p%s:%d", ProxyHost, ProxyPort);
	  if (strlen(ProxyUser))
	  {
		g_snprintf(pass, P_LENGTH*2+3, "-u%s:%s", ProxyUser, ProxyPass);
	  }
    }

#ifndef THREAD_SMSSEND
    args[0] = gnome_is_program_in_path (SMSEXECNAME);
    if (!args[0])
    {
	  g_printerr(_("SmsSend not found!\n"));
	  gnome_appbar_set_status(appbar, _("SmsSend not found!"));
	  gnome_appbar_set_progress(appbar, 0.0);
    	  pthread_exit((void *)(-1));
          /* Not reached  */
          return ((void *)(-1));
    }
    smsid = fork();
    if (smsid == 0)
#else
    args[0] = SMSEXECNAME;      /* not used, but it's more beautifull */
#endif
    {
	  /* child process when not defining THREAD_SMSSEND */
	  int argc=0, lastarg, i;
	  char *message = (char *)NULL;
	  char *value = (char *)NULL;
	  gnome_appbar_set_progress(appbar, 0.25);
	  /* create args array */
	  if (sms_param->Pv)
	  {
	      args[1] = sms_param->Pv->Filename;
	      argc = sms_param->Pv->NbParams;
	      for (i = 0; i < argc; i++)
	      {
		    if (g_strcasecmp(sms_param->Pv->Params[i].Name, "Message") == 0)
		    {
			  gchar **valuearray;
			  gchar *valuetmp;
		          /* change newline to space (use glib string arrays) */
		          GtkEditable *edit = GTK_EDITABLE(sms_param->Pv->Params[i].Widget);
		          valuetmp = gtk_editable_get_chars(edit, 0, gtk_editable_get_position(edit));
			  valuearray = g_strsplit(valuetmp,"\n",sms_param->Pv->Params[i].Size);
			  value = g_strjoinv(" ",valuearray);
		          /* enclose message in "" */
		          message = g_strdup_printf("\"%s\"", value);
		          args[2 + i] = message;
			  g_strfreev(valuearray);
	  	          g_free(valuetmp);
	  	          g_free(value);
		    }
		    else
		    {
			  value = gtk_entry_get_text(GTK_ENTRY(sms_param->Pv->Params[i].Widget));
			  if (KeepPass && g_strncasecmp(sms_param->Pv->Params[i].Name, "Password",10) == 0)
			  {
			    /* save password in preference file */
			    gchar *config_file;
			    gchar *path;
			    gchar *temp;

			    config_file=gnome_util_prepend_user_home(".smssend/smssendrc");
			    path = g_strdup_printf("=%s=/Passwords/%s",config_file,sms_param->Pv->Label);
			    if (value && strlen(value))
			    {
				temp = g_strdup(value);
				crypt_password(temp);
				gnome_config_set_string(path,temp);
			    	g_free(temp);
			    }
			    g_free(path);
			    g_free(config_file);
			    gnome_config_sync();
			  }
		          args[2 + i] = g_strdup(value);
			  g_free(value);
		    }
	      }
	  }
	  else if (sms_param->Help)
	  {
	      args[1] = (char *)arg_help;
	  }
	  else if (sms_param->Update)
	  {
	      args[1] = (char *)arg_update;
	  }
          lastarg = argc + 2;
	  if (UseProxy)
	  {
		/* don't forget the '--' ! */
		args[lastarg++] = (char *)arg_minus;
		args[lastarg++] = proxy;
		args[lastarg++] = pass;
	  }
	  /*if (DebugLevel) // Allways add my debug level */
	  {
	  	g_snprintf(debug, 10, "-d%d", DebugLevel);
		args[lastarg++] = debug;
	  }
	  if (TimeOut)
	  {
	  	g_snprintf(timeout, 5, "-t%d", TimeOut);
		args[lastarg++] = timeout;
	  }
	  args[lastarg] = NULL;
	  gnome_appbar_set_progress(appbar, 0.50);
#ifndef THREAD_SMSSEND
	  /* call smssend */
	  execvp(SMSEXECNAME, args);
          /* if exec() returns, there is something wrong */
	  g_printerr(_("SmsSend exec failed!\n"));
	  gnome_appbar_set_status(appbar, _("SmsSend exec failed!"));
#else
          /* call local version of smssend */
          smsstatus = abs(main_smssend(lastarg,args));
          gnome_appbar_set_progress(appbar, 0.75);
#endif
	  /* free all allocated strings */
	  if (message)
	  {
	      g_free(message);
	  }
	  for (i = 0; i < argc; i++)
	  {
		g_free(args[2 + i]);
	  }
#ifndef THREAD_SMSSEND
	  gnome_appbar_set_progress(appbar, 0.0);
	  retval = -2;
          /* exit child. note the use of _exit() instead of exit() */
	  _exit(-2);
#endif
    }
#ifndef THREAD_SMSSEND
    /* wait for child process to end before exiting thread */
    if (smsid > 0)
    {
	  waitpid(smsid, NULL, 0);
	  gnome_appbar_set_progress(appbar, 1.0);
    }
    else
    {
	  g_printerr(_("SmsSend fork failed!\n"));
	  gnome_appbar_set_status(appbar, _("SmsSend fork failed!"));
	  gnome_appbar_set_progress(appbar, 0.0);
	  retval = -3;
    }
    g_free(args[0]);
#else
    g_printerr(_("SmsSend done!\n"));
    /* check return status */
    switch (smsstatus)
    {
        case 0:
          gnome_appbar_set_progress(appbar, 1.0);
          gnome_appbar_set_status(appbar, _("Ready"));
          break;
        case 1:
          g_printerr(_("Cannot connect to the host\n"));
          gnome_appbar_set_status(appbar, _("Cannot connect to the host"));
          break;
        case 2:
          g_printerr(_("Connection timed out\n"));
          gnome_appbar_set_status(appbar, _("Connection timed out"));
          break;
        case 3:
          /* cant happen */
          g_printerr(_("Provider not specified\n"));
          gnome_appbar_set_status(appbar, _("Provider not specified"));
          break;
        case 4:
          g_printerr(_("Error in provider script\n"));
          gnome_appbar_set_status(appbar, _("Error in provider script"));
          break;
        case 5:
          g_printerr(_("Arguments list requested\n"));
          gnome_appbar_set_status(appbar, _("Arguments list requested"));
          break;
        case 6:
          g_printerr(_("Missing Provider arguments\n"));
          gnome_appbar_set_status(appbar, _("Missing Provider arguments"));
          break;
        case 7:
          g_printerr(_("Message is too long\n"));
          gnome_appbar_set_status(appbar, _("Message is too long"));
          break;
        case 8:
          g_printerr(_("Substitution error during execution\n"));
          gnome_appbar_set_status(appbar, _("Substitution error during execution"));
          break;
        default:
          g_printerr(_("Unknown error\n"));
          gnome_appbar_set_status(appbar, _("Unknown error"));
          break;
    }
    /* TODO:
     * maybe use the GTK/GNOME html widgets to look at the dumped html
     * files when (DebugLevel & DUMP) is true ?
     * May need to modify skylib to use a filename based on provider filename.
     * Multi-thread problem: prefix DEBUG messages with provider filename.
     */
#endif
    g_free(sms_param);
    pthread_exit((void *)retval);
    /* Not reached  */
    return ((void *)retval);
}
#undef gnome_appbar_set_status
#undef gnome_appbar_set_progress

void launch_sms(PProvider_t Pv, gboolean Help, gboolean Update)
{
    /* prepare thread to launch smssend */
    GnomeAppBar *appbar = GNOME_APPBAR(lookup_widget(gnome_smssend, "appbar"));
    /* allocate parameters struct */
    PcmdParam_t sms_param = g_new(cmdParam_t, 1);
    /* thread stuff */
    pthread_t      thread_sms;
    int            thread_status;

    sms_param->Pv     = Pv;
    sms_param->Help   = Help;
    sms_param->Update = Update;

    gnome_appbar_set_status(appbar, "SmsSend ...");
    gnome_appbar_set_progress(appbar, 0.0);
#ifndef THREAD_SMSSEND
    /* attach a 'ignore' signal handler to SIGCHLD to avoid zombies */
    signal(SIGCHLD,SIG_IGN);
#endif

    thread_status = pthread_create(&thread_sms, NULL, DoSms, (void *)sms_param);
    if (thread_status < 0)
       g_printerr(_("Error creating SmsSend thread!\n"));
}
